import os
from typing import Optional, Type


def env_var(
    name: str,
    type_: Optional[Type] = str,
    separator: Optional[str] = ",",
) -> str | list | bool | int:
    """Get environment variable

    Parameters:
        var (str): the var to get
        type_ (Type): the kind of value you expect to retrieve from var
        separator (str):  if getting list, which separator to use

    Returns:
        value of env var
    """
    value = os.environ.get(name)
    if not value:
        raise KeyError(f"Missing required environment variable for '{name}'.")

    allowed_types = [int, str, list, bool]
    if type_ not in allowed_types:
        raise ValueError(
            f"Type {type_} is not allowed. Use one of {', '.join(allowed_types)}"
        )

    if type_ == str:
        return value

    if type_ == list:
        try:
            return [item.strip() for item in value.split(separator)]
        except Exception as e:
            raise ValueError(f"Error parsing list from env var '{name}': {e}")

    if type_ == bool:
        if value.upper() == "TRUE":
            return True
        if value.upper() == "FALSE":
            return False

        raise ValueError(
            f"Bool must be set to true or false (case insensitive), not: '{value}'"
        )

    if type_ == int:
        if value.isnumeric():
            return int(value)

        raise ValueError(f"Int must be set to a valid integer, not: '{value}'")



import os
from roskarl.env import env_var
from unittest import TestCase


class TestGetEnvVar(TestCase):
    def test_get_env_var_string(self):
        env_var_value = "hello"
        env_var_name = "x"

        os.environ[env_var_name] = env_var_value

        real_value = env_var(name=env_var_name)

        assert env_var_value == real_value

    def test_get_env_var_empty(self):
        env_var_name = "this_var_isnt_set_at_all"

        with self.assertRaises(KeyError):
            env_var(name=env_var_name)

    def test_get_int_var_valid_value(self):
        env_var_value = "5"
        env_var_name = "x"
        os.environ[env_var_name] = env_var_value

        real_value = env_var(name=env_var_name, type_=int)

        assert type(real_value) == type(529835)

    def test_get_int_var_invalid_value(self):
        env_var_value = "5aioaiwdm"
        env_var_name = "x"
        env_type_ = int
        os.environ[env_var_name] = env_var_value

        with self.assertRaises(ValueError):
            env_var(name=env_var_name, type_=env_type_)

    def test_get_bool_var_valid_value_true(self):
        env_var_value = "true"
        env_var_name = "x"
        os.environ[env_var_name] = env_var_value

        real_value = env_var(name=env_var_name, type_=bool)

        assert real_value == True

    def test_get_bool_var_valid_value_false_chaos_case(self):
        env_var_value = "FaLsE"
        env_var_name = "x"
        os.environ[env_var_name] = env_var_value

        real_value = env_var(name=env_var_name, type_=bool)

        assert real_value == False

    def test_get_bool_var_valid_value_invalid_value(self):
        env_var_value = "do ducks wear jackets"
        env_var_name = "x"
        env_type_ = bool
        os.environ[env_var_name] = env_var_value

        with self.assertRaises(ValueError):
            env_var(name=env_var_name, type_=env_type_)

    def test_get_list_var(self):
        env_var_value = "A,B,C,D"
        env_var_name = "proper_list_yo"
        env_type_ = list
        os.environ[env_var_name] = env_var_value

        real_value = env_var(name=env_var_name, type_=env_type_)

        assert real_value == ["A", "B", "C", "D"]

    def test_get_list_var_with_separator(self):
        env_var_value = "A|B|C|D"
        env_var_name = "x"
        env_type_ = list
        separator = "|"
        os.environ[env_var_name] = env_var_value

        real_value = env_var(name=env_var_name, type_=env_type_, separator=separator)

        assert real_value == ["A", "B", "C", "D"]
